@@ -21,11 +21,16 @@
 
 #define SIP_NO_FILE
 #include "qgsprocessingwidgetwrapper.h"
+#include "qgsmaptool.h"
+
+#include <QAbstractButton>
 
 class QCheckBox;
 class QComboBox;
 class QLineEdit;
 class QPlainTextEdit;
+class QToolButton;
+class QButtonGroup;
 class QgsProjectionSelectionWidget;
 class QgsSpinBox;
 class QgsDoubleSpinBox;
@@ -34,6 +39,13 @@ class QgsProcessingMatrixParameterPanel;
 class QgsFileWidget;
 class QgsFieldExpressionWidget;
 class QgsExpressionLineEdit;
+class QgsProcessingParameterEnum;
+class QgsLayoutComboBox;
+class QgsLayoutItemComboBox;
+class QgsPrintLayout;
+class QgsScaleWidget;
+class QgsSnapIndicator;
+class QgsFilterLineEdit;
 
 ///@cond PRIVATE
 
@@ -256,6 +268,34 @@ class GUI_EXPORT QgsProcessingDistanceWidgetWrapper : public QgsProcessingNumeri
 };
 
 
+class GUI_EXPORT QgsProcessingScaleWidgetWrapper : public QgsProcessingNumericWidgetWrapper
+{
+    Q_OBJECT
+
+  public:
+
+    QgsProcessingScaleWidgetWrapper( const QgsProcessingParameterDefinition *parameter = nullptr,
+                                     QgsProcessingGui::WidgetType type = QgsProcessingGui::Standard, QWidget *parent = nullptr );
+
+    // QgsProcessingParameterWidgetFactoryInterface
+    QString parameterType() const override;
+    QgsAbstractProcessingParameterWidgetWrapper *createWidgetWrapper( const QgsProcessingParameterDefinition *parameter, QgsProcessingGui::WidgetType type ) override;
+
+    // QgsProcessingParameterWidgetWrapper interface
+    QWidget *createWidget() override SIP_FACTORY;
+    void setWidgetContext( const QgsProcessingParameterWidgetContext &context ) override;
+  protected:
+
+    QVariant widgetValue() const override;
+    void setWidgetValue( const QVariant &value, QgsProcessingContext &context ) override;
+
+  private:
+
+    QgsScaleWidget *mScaleWidget = nullptr;
+
+    friend class TestProcessingGui;
+};
+
 class GUI_EXPORT QgsProcessingRangeWidgetWrapper : public QgsAbstractProcessingParameterWidgetWrapper, public QgsProcessingParameterWidgetFactoryInterface
 {
     Q_OBJECT
@@ -401,6 +441,282 @@ class GUI_EXPORT QgsProcessingExpressionWidgetWrapper : public QgsAbstractProces
     friend class TestProcessingGui;
 };
 
+
+class GUI_EXPORT QgsProcessingEnumCheckboxPanelWidget : public QWidget
+{
+    Q_OBJECT
+
+  public:
+
+    QgsProcessingEnumCheckboxPanelWidget( QWidget *parent = nullptr, const QgsProcessingParameterEnum *param = nullptr, int columns = 2 );
+    QVariant value() const;
+    void setValue( const QVariant &value );
+
+  signals:
+
+    void changed();
+
+  private slots:
+
+    void showPopupMenu();
+    void selectAll();
+    void deselectAll();
+
+  private:
+
+    const QgsProcessingParameterEnum *mParam = nullptr;
+    QMap< QVariant, QAbstractButton * > mButtons;
+    QButtonGroup *mButtonGroup = nullptr;
+    int mColumns = 2;
+    bool mBlockChangedSignal = false;
+
+    friend class TestProcessingGui;
+};
+
+class GUI_EXPORT QgsProcessingEnumPanelWidget : public QWidget
+{
+    Q_OBJECT
+
+  public:
+
+    QgsProcessingEnumPanelWidget( QWidget *parent = nullptr, const QgsProcessingParameterEnum *param = nullptr );
+    QVariant value() const { return mValue; }
+    void setValue( const QVariant &value );
+
+  signals:
+
+    void changed();
+
+  private slots:
+
+    void showDialog();
+
+  private:
+
+    void updateSummaryText();
+
+    const QgsProcessingParameterEnum *mParam = nullptr;
+    QLineEdit *mLineEdit = nullptr;
+    QToolButton *mToolButton = nullptr;
+
+    QVariantList mValue;
+
+    friend class TestProcessingGui;
+};
+
+
+class GUI_EXPORT QgsProcessingEnumWidgetWrapper : public QgsAbstractProcessingParameterWidgetWrapper, public QgsProcessingParameterWidgetFactoryInterface
+{
+    Q_OBJECT
+
+  public:
+
+    QgsProcessingEnumWidgetWrapper( const QgsProcessingParameterDefinition *parameter = nullptr,
+                                    QgsProcessingGui::WidgetType type = QgsProcessingGui::Standard, QWidget *parent = nullptr );
+
+    // QgsProcessingParameterWidgetFactoryInterface
+    QString parameterType() const override;
+    QgsAbstractProcessingParameterWidgetWrapper *createWidgetWrapper( const QgsProcessingParameterDefinition *parameter, QgsProcessingGui::WidgetType type ) override;
+
+    // QgsProcessingParameterWidgetWrapper interface
+    QWidget *createWidget() override SIP_FACTORY;
+
+  protected:
+
+    void setWidgetValue( const QVariant &value, QgsProcessingContext &context ) override;
+    QVariant widgetValue() const override;
+
+    QStringList compatibleParameterTypes() const override;
+
+    QStringList compatibleOutputTypes() const override;
+
+    QList< int > compatibleDataTypes() const override;
+    QString modelerExpressionFormatString() const override;
+  private:
+
+    QComboBox *mComboBox = nullptr;
+    QgsProcessingEnumPanelWidget *mPanel = nullptr;
+    QgsProcessingEnumCheckboxPanelWidget *mCheckboxPanel = nullptr;
+
+    friend class TestProcessingGui;
+};
+
+
+
+class GUI_EXPORT QgsProcessingLayoutWidgetWrapper : public QgsAbstractProcessingParameterWidgetWrapper, public QgsProcessingParameterWidgetFactoryInterface
+{
+    Q_OBJECT
+
+  public:
+
+    QgsProcessingLayoutWidgetWrapper( const QgsProcessingParameterDefinition *parameter = nullptr,
+                                      QgsProcessingGui::WidgetType type = QgsProcessingGui::Standard, QWidget *parent = nullptr );
+
+    // QgsProcessingParameterWidgetFactoryInterface
+    QString parameterType() const override;
+    QgsAbstractProcessingParameterWidgetWrapper *createWidgetWrapper( const QgsProcessingParameterDefinition *parameter, QgsProcessingGui::WidgetType type ) override;
+
+    // QgsProcessingParameterWidgetWrapper interface
+    QWidget *createWidget() override SIP_FACTORY;
+
+  protected:
+
+    void setWidgetValue( const QVariant &value, QgsProcessingContext &context ) override;
+    QVariant widgetValue() const override;
+
+    QStringList compatibleParameterTypes() const override;
+
+    QStringList compatibleOutputTypes() const override;
+
+    QList< int > compatibleDataTypes() const override;
+    QString modelerExpressionFormatString() const override;
+  private:
+
+    QgsLayoutComboBox *mComboBox = nullptr;
+    QLineEdit *mLineEdit = nullptr;
+
+    friend class TestProcessingGui;
+};
+
+
+class GUI_EXPORT QgsProcessingLayoutItemWidgetWrapper : public QgsAbstractProcessingParameterWidgetWrapper, public QgsProcessingParameterWidgetFactoryInterface
+{
+    Q_OBJECT
+
+  public:
+
+    QgsProcessingLayoutItemWidgetWrapper( const QgsProcessingParameterDefinition *parameter = nullptr,
+                                          QgsProcessingGui::WidgetType type = QgsProcessingGui::Standard, QWidget *parent = nullptr );
+
+    // QgsProcessingParameterWidgetFactoryInterface
+    QString parameterType() const override;
+    QgsAbstractProcessingParameterWidgetWrapper *createWidgetWrapper( const QgsProcessingParameterDefinition *parameter, QgsProcessingGui::WidgetType type ) override;
+
+    // QgsProcessingParameterWidgetWrapper interface
+    QWidget *createWidget() override SIP_FACTORY;
+    void postInitialize( const QList< QgsAbstractProcessingParameterWidgetWrapper * > &wrappers ) override;
+
+
+  public slots:
+    void setLayoutParameterValue( const QVariant &value );
+    void setLayout( QgsPrintLayout *layout );
+
+  protected:
+
+    void setWidgetValue( const QVariant &value, QgsProcessingContext &context ) override;
+    QVariant widgetValue() const override;
+
+    QStringList compatibleParameterTypes() const override;
+
+    QStringList compatibleOutputTypes() const override;
+
+    QList< int > compatibleDataTypes() const override;
+    QString modelerExpressionFormatString() const override;
+  private:
+
+    QgsLayoutItemComboBox *mComboBox = nullptr;
+    QLineEdit *mLineEdit = nullptr;
+
+    friend class TestProcessingGui;
+};
+
+class GUI_EXPORT QgsProcessingPointMapTool : public QgsMapTool
+{
+    Q_OBJECT
+  public:
+    QgsProcessingPointMapTool( QgsMapCanvas *canvas );
+    ~QgsProcessingPointMapTool() override;
+    void deactivate() override;
+    void canvasMoveEvent( QgsMapMouseEvent *e ) override;
+    void canvasPressEvent( QgsMapMouseEvent *e ) override;
+    void keyPressEvent( QKeyEvent *e ) override;
+
+  signals:
+
+    void clicked( const QgsPointXY &point );
+    void complete();
+
+  private:
+
+    std::unique_ptr<QgsSnapIndicator> mSnapIndicator;
+    friend class TestProcessingGui;
+};
+
+class GUI_EXPORT QgsProcessingPointPanel : public QWidget
+{
+    Q_OBJECT
+
+  public:
+
+    QgsProcessingPointPanel( QWidget *parent );
+    void setMapCanvas( QgsMapCanvas *canvas );
+    void setAllowNull( bool allowNull );
+
+    QVariant value() const;
+    void clear();
+    void setValue( const QgsPointXY &point, const QgsCoordinateReferenceSystem &crs );
+
+  signals:
+
+    void toggleDialogVisibility( bool visible );
+    void changed();
+
+  private slots:
+
+    void selectOnCanvas();
+    void updatePoint( const QgsPointXY &point );
+    void pointPicked();
+
+  private:
+
+    QgsFilterLineEdit *mLineEdit = nullptr;
+    QToolButton *mButton = nullptr;
+    QgsMapCanvas *mCanvas = nullptr;
+    QgsCoordinateReferenceSystem mCrs;
+    QPointer< QgsMapTool > mPrevTool;
+    std::unique_ptr< QgsProcessingPointMapTool > mTool;
+    friend class TestProcessingGui;
+};
+
+
+class GUI_EXPORT QgsProcessingPointWidgetWrapper : public QgsAbstractProcessingParameterWidgetWrapper, public QgsProcessingParameterWidgetFactoryInterface
+{
+    Q_OBJECT
+
+  public:
+
+    QgsProcessingPointWidgetWrapper( const QgsProcessingParameterDefinition *parameter = nullptr,
+                                     QgsProcessingGui::WidgetType type = QgsProcessingGui::Standard, QWidget *parent = nullptr );
+
+    // QgsProcessingParameterWidgetFactoryInterface
+    QString parameterType() const override;
+    QgsAbstractProcessingParameterWidgetWrapper *createWidgetWrapper( const QgsProcessingParameterDefinition *parameter, QgsProcessingGui::WidgetType type ) override;
+
+    // QgsProcessingParameterWidgetWrapper interface
+    QWidget *createWidget() override SIP_FACTORY;
+    void setWidgetContext( const QgsProcessingParameterWidgetContext &context ) override;
+    void setDialog( QDialog *dialog ) override;
+
+  protected:
+
+    void setWidgetValue( const QVariant &value, QgsProcessingContext &context ) override;
+    QVariant widgetValue() const override;
+
+    QStringList compatibleParameterTypes() const override;
+
+    QStringList compatibleOutputTypes() const override;
+
+    QList< int > compatibleDataTypes() const override;
+    QString modelerExpressionFormatString() const override;
+  private:
+
+    QgsProcessingPointPanel *mPanel = nullptr;
+    QLineEdit *mLineEdit = nullptr;
+    QDialog *mDialog = nullptr;
+
+    friend class TestProcessingGui;
+};
+
 ///@endcond PRIVATE
 
 #endif // QGSPROCESSINGWIDGETWRAPPERIMPL_H
