@@ -19,7 +19,7 @@
 #define DEFAULT_SCALE_METHOD              QgsSymbol::ScaleDiameter
 
 #include "qgis_core.h"
-#include "qgis.h"
+// #include "qgis.h"
 #include <QColor>
 #include <QMap>
 #include <QPointF>
@@ -70,6 +70,8 @@ class CORE_EXPORT QgsSymbolLayer
           sipType = sipType_QgsFilledMarkerSymbolLayer;
         else if ( sipCpp->layerType() == "SvgMarker" )
           sipType = sipType_QgsSvgMarkerSymbolLayer;
+        else if ( sipCpp->layerType() == "RasterMarker" )
+          sipType = sipType_QgsRasterMarkerSymbolLayer;
         else if ( sipCpp->layerType() == "VectorField" )
           sipType = sipType_QgsVectorFieldSymbolLayer;
         else
@@ -137,8 +139,8 @@ class CORE_EXPORT QgsSymbolLayer
       PropertyFillStyle, //!< Fill style (eg solid, dots)
       PropertyJoinStyle, //!< Line join style
       PropertySecondaryColor, //!< Secondary color (eg for gradient fills)
-      PropertyLineAngle, //!< Line angle
-      PropertyLineDistance, //!< Distance between lines
+      PropertyLineAngle, //!< Line angle, or angle of hash lines for hash line symbols
+      PropertyLineDistance, //!< Distance between lines, or length of lines for hash line symbols
       PropertyGradientType, //!< Gradient fill type
       PropertyCoordinateMode, //!< Gradient coordinate mode
       PropertyGradientSpread, //!< Gradient spread mode
@@ -163,6 +165,7 @@ class CORE_EXPORT QgsSymbolLayer
       PropertyPlacement, //!< Line marker placement
       PropertyInterval, //!< Line marker interval
       PropertyOffsetAlongLine, //!< Offset along line
+      PropertyAverageAngleLength, //!< Length to average symbol angles over
       PropertyHorizontalAnchor, //!< Horizontal anchor point
       PropertyVerticalAnchor, //!< Vertical anchor point
       PropertyLayerEnabled, //!< Whether symbol layer is enabled
@@ -172,6 +175,8 @@ class CORE_EXPORT QgsSymbolLayer
       PropertyArrowHeadThickness, //!< Arrow head thickness
       PropertyArrowHeadType, //!< Arrow head type
       PropertyArrowType, //!< Arrow type
+      PropertyOffsetX, //!< Horizontal offset
+      PropertyOffsetY, //!< Vertical offset
     };
 
     /**
@@ -182,8 +187,14 @@ class CORE_EXPORT QgsSymbolLayer
 
     virtual ~QgsSymbolLayer();
 
+    //! QgsSymbolLayer cannot be copied
+    QgsSymbolLayer( const QgsSymbolLayer &other ) = delete;
+
+    //! QgsSymbolLayer cannot be copied
+    QgsSymbolLayer &operator=( const QgsSymbolLayer &other ) = delete;
+
     /**
-     * Returns true if symbol layer is enabled and will be drawn.
+     * Returns TRUE if symbol layer is enabled and will be drawn.
      * \see setEnabled()
      * \since QGIS 3.0
      */
@@ -211,7 +222,7 @@ class CORE_EXPORT QgsSymbolLayer
     /**
      * Set stroke color. Supported by marker and fill layers.
      * \since QGIS 2.1 */
-    virtual void setStrokeColor( const QColor &color ) { Q_UNUSED( color ); }
+    virtual void setStrokeColor( const QColor &color ) { Q_UNUSED( color ) }
 
     /**
      * Gets stroke color. Supported by marker and fill layers.
@@ -221,7 +232,7 @@ class CORE_EXPORT QgsSymbolLayer
     /**
      * Set fill color. Supported by marker and fill layers.
      * \since QGIS 2.1 */
-    virtual void setFillColor( const QColor &color ) { Q_UNUSED( color ); }
+    virtual void setFillColor( const QColor &color ) { Q_UNUSED( color ) }
 
     /**
      * Gets fill color. Supported by marker and fill layers.
@@ -243,9 +254,9 @@ class CORE_EXPORT QgsSymbolLayer
     virtual QgsSymbolLayer *clone() const = 0 SIP_FACTORY;
 
     virtual void toSld( QDomDocument &doc, QDomElement &element, const QgsStringMap &props ) const
-    { Q_UNUSED( props ); element.appendChild( doc.createComment( QStringLiteral( "SymbolLayerV2 %1 not implemented yet" ).arg( layerType() ) ) ); }
+    { Q_UNUSED( props ) element.appendChild( doc.createComment( QStringLiteral( "SymbolLayerV2 %1 not implemented yet" ).arg( layerType() ) ) ); }
 
-    virtual QString ogrFeatureStyle( double mmScaleFactor, double mapUnitScaleFactor ) const { Q_UNUSED( mmScaleFactor ); Q_UNUSED( mapUnitScaleFactor ); return QString(); }
+    virtual QString ogrFeatureStyle( double mmScaleFactor, double mapUnitScaleFactor ) const { Q_UNUSED( mmScaleFactor ) Q_UNUSED( mapUnitScaleFactor ); return QString(); }
 
     /**
      * Should be reimplemented by subclasses to return a string map that
@@ -278,7 +289,7 @@ class CORE_EXPORT QgsSymbolLayer
       drawn with an stroke will draw half the width
       of the stroke outside of the polygon. This amount is estimated, since it may
       be affected by data defined symbology rules.*/
-    virtual double estimateMaxBleed( const QgsRenderContext &context ) const { Q_UNUSED( context ); return 0; }
+    virtual double estimateMaxBleed( const QgsRenderContext &context ) const { Q_UNUSED( context ) return 0; }
 
     /**
      * Sets the units to use for sizes and widths within the symbol layer. Individual
@@ -288,7 +299,7 @@ class CORE_EXPORT QgsSymbolLayer
      * \param unit output units
      * \see outputUnit()
      */
-    virtual void setOutputUnit( QgsUnitTypes::RenderUnit unit ) { Q_UNUSED( unit ); }
+    virtual void setOutputUnit( QgsUnitTypes::RenderUnit unit ) { Q_UNUSED( unit ) }
 
     /**
      * Returns the units to use for sizes and widths within the symbol layer. Individual
@@ -300,12 +311,24 @@ class CORE_EXPORT QgsSymbolLayer
      */
     virtual QgsUnitTypes::RenderUnit outputUnit() const { return QgsUnitTypes::RenderUnknownUnit; }
 
-    virtual void setMapUnitScale( const QgsMapUnitScale &scale ) { Q_UNUSED( scale ); }
+    virtual void setMapUnitScale( const QgsMapUnitScale &scale ) { Q_UNUSED( scale ) }
     virtual QgsMapUnitScale mapUnitScale() const { return QgsMapUnitScale(); }
 
-    // used only with rending with symbol levels is turned on (0 = first pass, 1 = second, ...)
-    void setRenderingPass( int renderingPass ) { mRenderingPass = renderingPass; }
-    int renderingPass() const { return mRenderingPass; }
+    /**
+     * Specifies the rendering pass in which this symbol layer should be rendered.
+     * The lower the number, the lower the symbol will be rendered.
+     * 0: first pass, 1: second pass, ...
+     * Defaults to 0
+     */
+    void setRenderingPass( int renderingPass );
+
+    /**
+     * Specifies the rendering pass in which this symbol layer should be rendered.
+     * The lower the number, the lower the symbol will be rendered.
+     * 0: first pass, 1: second pass, ...
+     * Defaults to 0
+     */
+    int renderingPass() const;
 
     /**
      * Returns the set of attributes referenced by the layer. This includes attributes
@@ -317,6 +340,7 @@ class CORE_EXPORT QgsSymbolLayer
      * Sets a data defined property for the layer. Any existing property with the same key
      * will be overwritten.
      * \see dataDefinedProperties()
+     * \see Property
      * \since QGIS 3.0
      */
     virtual void setDataDefinedProperty( Property key, const QgsProperty &property );
@@ -375,6 +399,7 @@ class CORE_EXPORT QgsSymbolLayer
     /**
      * Returns a reference to the symbol layer's property collection, used for data defined overrides.
      * \see setDataDefinedProperties()
+     * \see Property
      * \since QGIS 3.0
      */
     QgsPropertyCollection &dataDefinedProperties() { return mDataDefinedProperties; }
@@ -394,6 +419,13 @@ class CORE_EXPORT QgsSymbolLayer
      */
     void setDataDefinedProperties( const QgsPropertyCollection &collection ) { mDataDefinedProperties = collection; }
 
+    /**
+     * Returns TRUE if the symbol layer (or any of its sub-symbols) contains data defined properties.
+     *
+     * \since QGIS 3.4.5
+     */
+    virtual bool hasDataDefinedProperties() const;
+
   protected:
 
     QgsSymbolLayer( QgsSymbol::SymbolType type, bool locked = false );
@@ -401,15 +433,15 @@ class CORE_EXPORT QgsSymbolLayer
     QgsSymbol::SymbolType mType;
 
     //! True if layer is enabled and should be drawn
-    bool mEnabled;
+    bool mEnabled = true;
 
-    bool mLocked;
+    bool mLocked = false;
     QColor mColor;
-    int mRenderingPass;
+    int mRenderingPass = 0;
 
     QgsPropertyCollection mDataDefinedProperties;
 
-    QgsPaintEffect *mPaintEffect = nullptr;
+    std::unique_ptr< QgsPaintEffect > mPaintEffect;
     QgsFields mFields;
 
     // Configuration of selected symbology implementation
@@ -445,6 +477,10 @@ class CORE_EXPORT QgsSymbolLayer
     //! Property definitions
     static QgsPropertiesDefinition sPropertyDefinitions;
 
+#ifdef SIP_RUN
+    QgsSymbolLayer( const QgsSymbolLayer &other );
+#endif
+
 };
 
 //////////////////////
@@ -474,8 +510,16 @@ class CORE_EXPORT QgsMarkerSymbolLayer : public QgsSymbolLayer
       Bottom, //!< Align to bottom of symbol
     };
 
+    //! QgsMarkerSymbolLayer cannot be copied
+    QgsMarkerSymbolLayer( const QgsMarkerSymbolLayer &other ) = delete;
+
+    //! QgsMarkerSymbolLayer cannot be copied
+    QgsMarkerSymbolLayer &operator=( const QgsMarkerSymbolLayer &other ) = delete;
+
     void startRender( QgsSymbolRenderContext &context ) override;
 
+    void stopRender( QgsSymbolRenderContext &context ) override;
+
     /**
      * Renders a marker at the specified point. Derived classes must implement this to
      * handle drawing the point.
@@ -518,7 +562,7 @@ class CORE_EXPORT QgsMarkerSymbolLayer : public QgsSymbolLayer
      * \see setSizeUnit()
      * \see setSizeMapUnitScale()
      */
-    void setSize( double size ) { mSize = size; }
+    virtual void setSize( double size ) { mSize = size; }
 
     /**
      * Returns the symbol size. Units are specified by sizeUnit().
@@ -671,7 +715,7 @@ class CORE_EXPORT QgsMarkerSymbolLayer : public QgsSymbolLayer
      * \param props symbol layer definition (see properties())
      */
     virtual void writeSldMarker( QDomDocument &doc, QDomElement &element, const QgsStringMap &props ) const
-    { Q_UNUSED( props ); element.appendChild( doc.createComment( QStringLiteral( "QgsMarkerSymbolLayer %1 not implemented yet" ).arg( layerType() ) ) ); }
+    { Q_UNUSED( props ) element.appendChild( doc.createComment( QStringLiteral( "QgsMarkerSymbolLayer %1 not implemented yet" ).arg( layerType() ) ) ); }
 
     void setOutputUnit( QgsUnitTypes::RenderUnit unit ) override;
     QgsUnitTypes::RenderUnit outputUnit() const override;
@@ -690,7 +734,7 @@ class CORE_EXPORT QgsMarkerSymbolLayer : public QgsSymbolLayer
 
     /**
      * Constructor for QgsMarkerSymbolLayer.
-     * \param locked set to true to lock symbol color
+     * \param locked set to TRUE to lock symbol color
      */
     QgsMarkerSymbolLayer( bool locked = false );
 
@@ -755,6 +799,10 @@ class CORE_EXPORT QgsMarkerSymbolLayer : public QgsSymbolLayer
   private:
     static QgsMarkerSymbolLayer::HorizontalAnchorPoint decodeHorizontalAnchorPoint( const QString &str );
     static QgsMarkerSymbolLayer::VerticalAnchorPoint decodeVerticalAnchorPoint( const QString &str );
+
+#ifdef SIP_RUN
+    QgsMarkerSymbolLayer( const QgsMarkerSymbolLayer &other );
+#endif
 };
 
 /**
@@ -764,16 +812,139 @@ class CORE_EXPORT QgsMarkerSymbolLayer : public QgsSymbolLayer
 class CORE_EXPORT QgsLineSymbolLayer : public QgsSymbolLayer
 {
   public:
+
+    //! Options for filtering rings when the line symbol layer is being used to render a polygon's rings.
+    enum RenderRingFilter
+    {
+      AllRings, //!< Render both exterior and interior rings
+      ExteriorRingOnly, //!< Render the exterior ring only
+      InteriorRingsOnly, //!< Render the interior rings only
+    };
+
+    //! QgsLineSymbolLayer cannot be copied
+    QgsLineSymbolLayer( const QgsLineSymbolLayer &other ) = delete;
+
+    //! QgsLineSymbolLayer cannot be copied
+    QgsLineSymbolLayer &operator=( const QgsLineSymbolLayer &other ) = delete;
+
+    void setOutputUnit( QgsUnitTypes::RenderUnit unit ) override;
+    QgsUnitTypes::RenderUnit outputUnit() const override;
+    void setMapUnitScale( const QgsMapUnitScale &scale ) override;
+    QgsMapUnitScale mapUnitScale() const override;
+    void drawPreviewIcon( QgsSymbolRenderContext &context, QSize size ) override;
+    double dxfWidth( const QgsDxfExport &e, QgsSymbolRenderContext &context ) const override;
+
+    /**
+     * Renders the line symbol layer along the line joining \a points, using the given render \a context.
+     * \see renderPolygonStroke()
+     */
     virtual void renderPolyline( const QPolygonF &points, QgsSymbolRenderContext &context ) = 0;
 
+    /**
+     * Renders the line symbol layer along the outline of polygon, using the given render \a context.
+     *
+     * The exterior ring of the polygon is specified in \a points. Optionally, interior
+     * rings are set via the \a rings argument.
+     *
+     * \see renderPolyline()
+     */
     virtual void renderPolygonStroke( const QPolygonF &points, QList<QPolygonF> *rings, QgsSymbolRenderContext &context );
 
+    /**
+     * Sets the \a width of the line symbol layer.
+     *
+     * Calling this method updates the width of the line symbol layer, without
+     * changing the existing width units. It has different effects depending
+     * on the line symbol layer subclass, e.g. for a simple line layer it
+     * changes the stroke width of the line, for a marker line layer it
+     * changes the size of the markers used to draw the line.
+     *
+     * \see width()
+     * \warning Since the width units vary, this method is useful for changing the
+     * relative width of a line symbol layer only.
+     */
     virtual void setWidth( double width ) { mWidth = width; }
+
+    /**
+     * Returns the estimated width for the line symbol layer.
+     *
+     * \warning This returned value is inaccurate if the symbol layer has sub-symbols with
+     * different width units. Use the overload accepting a QgsRenderContext
+     * argument instead for accurate sizes in this case.
+     *
+     * \see setWidth()
+     */
     virtual double width() const { return mWidth; }
 
+    /**
+     * Returns the line symbol layer width, in painter units.
+     *
+     * This method returns an accurate width by calculating the actual rendered
+     * width of the symbol layer using the provided render \a context.
+     *
+     * \see setWidth()
+     *
+     * \since QGIS 3.4.5
+     */
+    virtual double width( const QgsRenderContext &context ) const;
+
+    /**
+     * Returns the line's offset.
+     *
+     * Offset units can be retrieved by calling offsetUnit().
+     *
+     * \see setOffset()
+     * \see offsetUnit()
+     * \see offsetMapUnitScale()
+     */
     double offset() const { return mOffset; }
+
+    /**
+     * Sets the line's \a offset.
+     *
+     * Offset units are set via setOffsetUnit().
+     *
+     * \see offset()
+     * \see setOffsetUnit()
+     * \see setOffsetMapUnitScale()
+     */
     void setOffset( double offset ) { mOffset = offset; }
 
+    /**
+     * Sets the \a unit for the line's offset.
+     * \see offsetUnit()
+     * \see setOffset()
+     * \see setOffsetMapUnitScale()
+    */
+    void setOffsetUnit( QgsUnitTypes::RenderUnit unit ) { mOffsetUnit = unit; }
+
+    /**
+     * Returns the units for the line's offset.
+     * \see setOffsetUnit()
+     * \see offset()
+     * \see offsetMapUnitScale()
+    */
+    QgsUnitTypes::RenderUnit offsetUnit() const { return mOffsetUnit; }
+
+    /**
+     * Sets the map unit \a scale for the line's offset.
+     * \see offsetMapUnitScale()
+     * \see setOffset()
+     * \see setOffsetUnit()
+    */
+    void setOffsetMapUnitScale( const QgsMapUnitScale &scale ) { mOffsetMapUnitScale = scale; }
+
+    /**
+     * Returns the map unit scale for the line's offset.
+     * \see setOffsetMapUnitScale()
+     * \see offset()
+     * \see offsetUnit()
+    */
+    const QgsMapUnitScale &offsetMapUnitScale() const { return mOffsetMapUnitScale; }
+
+    // TODO QGIS 4.0 - setWidthUnit(), widthUnit(), setWidthUnitScale(), widthUnitScale()
+    // only apply to simple line symbol layers and do not belong here.
+
     /**
      * Sets the units for the line's width.
      * \param unit width units
@@ -791,30 +962,28 @@ class CORE_EXPORT QgsLineSymbolLayer : public QgsSymbolLayer
     const QgsMapUnitScale &widthMapUnitScale() const { return mWidthMapUnitScale; }
 
     /**
-     * Sets the units for the line's offset.
-     * \param unit offset units
-     * \see offsetUnit()
-    */
-    void setOffsetUnit( QgsUnitTypes::RenderUnit unit ) { mOffsetUnit = unit; }
+     * Returns the line symbol layer's ring filter, which controls which rings are
+     * rendered when the line symbol is being used to draw a polygon's rings.
+     *
+     * This setting has no effect when the line symbol is not being rendered
+     * for a polygon.
+     *
+     * \see setRingFilter()
+     * \since QGIS 3.6
+     */
+    RenderRingFilter ringFilter() const;
 
     /**
-     * Returns the units for the line's offset.
-     * \see setOffsetUnit()
-    */
-    QgsUnitTypes::RenderUnit offsetUnit() const { return mOffsetUnit; }
-
-    void setOffsetMapUnitScale( const QgsMapUnitScale &scale ) { mOffsetMapUnitScale = scale; }
-    const QgsMapUnitScale &offsetMapUnitScale() const { return mOffsetMapUnitScale; }
-
-    void setOutputUnit( QgsUnitTypes::RenderUnit unit ) override;
-    QgsUnitTypes::RenderUnit outputUnit() const override;
-
-    void setMapUnitScale( const QgsMapUnitScale &scale ) override;
-    QgsMapUnitScale mapUnitScale() const override;
-
-    void drawPreviewIcon( QgsSymbolRenderContext &context, QSize size ) override;
-
-    double dxfWidth( const QgsDxfExport &e, QgsSymbolRenderContext &context ) const override;
+     * Sets the line symbol layer's ring \a filter, which controls which rings are
+     * rendered when the line symbol is being used to draw a polygon's rings.
+     *
+     * This setting has no effect when the line symbol is not being rendered
+     * for a polygon.
+     *
+     * \see ringFilter()
+     * \since QGIS 3.6
+     */
+    void setRingFilter( QgsLineSymbolLayer::RenderRingFilter filter );
 
   protected:
     QgsLineSymbolLayer( bool locked = false );
@@ -825,6 +994,13 @@ class CORE_EXPORT QgsLineSymbolLayer : public QgsSymbolLayer
     double mOffset = 0;
     QgsUnitTypes::RenderUnit mOffsetUnit = QgsUnitTypes::RenderMillimeters;
     QgsMapUnitScale mOffsetMapUnitScale;
+
+    RenderRingFilter mRingFilter = AllRings;
+
+  private:
+#ifdef SIP_RUN
+    QgsLineSymbolLayer( const QgsLineSymbolLayer &other );
+#endif
 };
 
 /**
@@ -834,6 +1010,13 @@ class CORE_EXPORT QgsLineSymbolLayer : public QgsSymbolLayer
 class CORE_EXPORT QgsFillSymbolLayer : public QgsSymbolLayer
 {
   public:
+
+    //! QgsFillSymbolLayer cannot be copied
+    QgsFillSymbolLayer( const QgsFillSymbolLayer &other ) = delete;
+
+    //! QgsFillSymbolLayer cannot be copied
+    QgsFillSymbolLayer &operator=( const QgsFillSymbolLayer &other ) = delete;
+
     virtual void renderPolygon( const QPolygonF &points, QList<QPolygonF> *rings, QgsSymbolRenderContext &context ) = 0;
 
     void drawPreviewIcon( QgsSymbolRenderContext &context, QSize size ) override;
@@ -847,6 +1030,11 @@ class CORE_EXPORT QgsFillSymbolLayer : public QgsSymbolLayer
     void _renderPolygon( QPainter *p, const QPolygonF &points, const QList<QPolygonF> *rings, QgsSymbolRenderContext &context );
 
     double mAngle = 0.0;
+
+  private:
+#ifdef SIP_RUN
+    QgsFillSymbolLayer( const QgsFillSymbolLayer &other );
+#endif
 };
 
 class QgsSymbolLayerWidget;  // why does SIP fail, when this isn't here
